@@ -156,6 +156,13 @@ static void mymergesort( int first, int last, Segment **seg )
 	static TLS int i, j, k, p;
 	static TLS int allo = 0;
 	static TLS Segment **work = NULL;
+
+	if( seg == NULL )
+	{
+		free( work ); work = NULL;
+		return;
+	}
+
 	if( last > allo )
 	{
 		allo = last;
@@ -225,6 +232,7 @@ double Fgetlag( char  **seq1, char  **seq2,
 	int len1, len2;
 	int totallen;
 	float dumfl = 0.0;
+	int headgp, tailgp;
 
 	len1 = strlen( seq1[0] );
 	len2 = strlen( seq2[0] );
@@ -405,7 +413,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 
 	for( j=0; j<n20or4or2; j++ )
 	{
-		fft( nlen, seqVector2[j], (j==0) );
+		fft( nlen, seqVector2[j], 0 );
 		fft( nlen, seqVector1[j], 0 );
 	}
 #if 0
@@ -670,6 +678,9 @@ system( "/usr/bin/gnuplot list.plot" );
 	for( j=0; j<clus2; j++ ) result2[j][0] = 0;
 	for( i=0; i<count-1; i++ )
 	{
+		if( i == 0 ) headgp = outgap; else headgp = 1;
+		if( i == count-2 ) tailgp = outgap; else tailgp = 1;
+		
 #if DEBUG
 		fprintf( stderr, "DP %03d / %03d %4d to ", i+1, count-1, totallen );
 #else
@@ -693,23 +704,23 @@ system( "/usr/bin/gnuplot list.plot" );
 				Aalign( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen );
 				break;
 			case( 'M' ):
-					MSalignmm( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL );
+					MSalignmm( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, headgp, tailgp );
 				break;
 			case( 'A' ):
 				if( clus1 == 1 && clus2 == 1 )
-					G__align11( tmpres1, tmpres2, alloclen );
+					G__align11( tmpres1, tmpres2, alloclen, headgp, tailgp );
 				else
-					A__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
+					A__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, headgp, tailgp );
 				break;
 			case( 'H' ):
 				if( clus1 == 1 && clus2 == 1 )
-					G__align11( tmpres1, tmpres2, alloclen );
+					G__align11( tmpres1, tmpres2, alloclen, headgp, tailgp );
 				else
 					H__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
 				break;
 			case( 'Q' ):
 				if( clus1 == 1 && clus2 == 1 )
-					G__align11( tmpres1, tmpres2, alloclen );
+					G__align11( tmpres1, tmpres2, alloclen, headgp, tailgp );
 				else
 					Q__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
 				break;
@@ -760,11 +771,11 @@ system( "/usr/bin/gnuplot list.plot" );
 
 
 
-
 float Falign( char  **seq1, char  **seq2, 
 			  double *eff1, double *eff2, 
 			  int    clus1, int    clus2,
-			  int alloclen, int *fftlog )
+			  int alloclen, int *fftlog,
+			  int *chudanpt, int chudanref, int *chudanres )
 {
 	int i, j, k, l, m, maxk;
 	int nlen, nlen2, nlen4;
@@ -806,8 +817,59 @@ float Falign( char  **seq1, char  **seq2,
 	int totallen;
 	float totalscore;
 	float dumfl = 0.0;
+	int headgp, tailgp;
 
 
+	if( seq1 == NULL )
+	{
+		if( result1 ) 
+		{
+//			fprintf( stderr, "Freeing localarrays in Falign\n" );
+			localalloclen = 0;
+			mymergesort( 0, 0, NULL );
+			alignableReagion( 0, 0, NULL, NULL, NULL, NULL, NULL );
+			fft( 0, NULL, 1 );
+			A__align( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+			G__align11( NULL, NULL, 0, 0, 0 );
+			blockAlign2( NULL, NULL, NULL, NULL, NULL, NULL );
+			if( crossscore ) FreeDoubleMtx( crossscore );
+			FreeCharMtx( result1 );
+			FreeCharMtx( result2 );
+			FreeCharMtx( tmpres1 );
+			FreeCharMtx( tmpres2 );
+			FreeCharMtx( tmpseq1 );
+			FreeCharMtx( tmpseq2 );
+			free( sgap1 );
+			free( egap1 );
+			free( sgap2 );
+			free( egap2 );
+			free( kouho );
+			free( cut1 );
+			free( cut2 );
+			free( tmpptr1 );
+			free( tmpptr2 );
+			free( segment );
+			free( segment1 );
+			free( segment2 );
+			free( sortedseg1 );
+			free( sortedseg2 );
+			if( !kobetsubunkatsu )
+			{
+				FreeFukusosuuMtx ( seqVector1 );
+				FreeFukusosuuMtx ( seqVector2 );
+				FreeFukusosuuVec( naisekiNoWa );
+				FreeFukusosuuMtx( naiseki );
+				FreeDoubleVec( soukan );
+			}
+		}
+		else
+		{
+//			fprintf( stderr, "Did not allocate localarrays in Falign\n" );
+		}
+
+		return( 0.0 );
+	}
+
 	len1 = strlen( seq1[0] );
 	len2 = strlen( seq2[0] );
 	nlentmp = MAX( len1, len2 );
@@ -1016,7 +1078,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 
 		for( j=0; j<n20or4or2; j++ )
 		{
-			fft( nlen, seqVector2[j], (j==0) );
+			fft( nlen, seqVector2[j], 0 );
 			fft( nlen, seqVector1[j], 0 );
 		}
 #if 0
@@ -1333,6 +1395,9 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 	for( i=0; i<count-1; i++ )
 	{
 		*fftlog += 1;
+		if( i == 0 ) headgp = outgap; else headgp = 1;
+		if( i == count-2 ) tailgp = outgap; else tailgp = 1;
+
 
 		if( cut1[i] ) // chuui
 		{
@@ -1445,20 +1510,20 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 				totalscore += Aalign( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen );
 				break;
 			case( 'M' ):
-					totalscore += MSalignmm( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, sgap1, sgap2, egap1, egap2 );
+					totalscore += MSalignmm( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, sgap1, sgap2, egap1, egap2, chudanpt, chudanref, chudanres, headgp, tailgp );
 				break;
 			case( 'A' ):
 				if( clus1 == 1 && clus2 == 1 )
 				{
-					totalscore += G__align11( tmpres1, tmpres2, alloclen );
+					totalscore += G__align11( tmpres1, tmpres2, alloclen, headgp, tailgp );
 				}
 				else
-					totalscore += A__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, sgap1, sgap2, egap1, egap2 );
+					totalscore += A__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, sgap1, sgap2, egap1, egap2, chudanpt, chudanref, chudanres, headgp, tailgp );
 				break;
 			case( 'H' ):
 				if( clus1 == 1 && clus2 == 1 )
 				{
-					totalscore += G__align11( tmpres1, tmpres2, alloclen );
+					totalscore += G__align11( tmpres1, tmpres2, alloclen, headgp, tailgp );
 				}
 				else
 					totalscore += H__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, sgap1, sgap2, egap1, egap2 );
@@ -1466,7 +1531,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 			case( 'Q' ):
 				if( clus1 == 1 && clus2 == 1 )
 				{
-					totalscore += G__align11( tmpres1, tmpres2, alloclen );
+					totalscore += G__align11( tmpres1, tmpres2, alloclen, headgp, tailgp );
 				}
 				else
 					totalscore += Q__align( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, NULL, &dumfl, sgap1, sgap2, egap1, egap2 );
@@ -1477,6 +1542,14 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 				break;
 		}
 
+#ifdef enablemultithread
+		if( chudanres && *chudanres )
+		{
+//			fprintf( stderr, "\n\n## CHUUDAN!!! at Falign_localhom\n" );
+			return( -1.0 );
+		}
+#endif
+
 		nlen = strlen( tmpres1[0] );
 		if( totallen + nlen > alloclen )
 		{
@@ -1528,7 +1601,9 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 
 
 
-
+/*
+sakujo wo kentou (2010/10/05)
+*/
 float Falign_udpari_long( char  **seq1, char  **seq2, 
 			  double *eff1, double *eff2, 
 			  int    clus1, int    clus2,
@@ -1574,9 +1649,11 @@ float Falign_udpari_long( char  **seq1, char  **seq2,
 	int totallen;
 	float totalscore;
 	int nkouho;
+	int headgp, tailgp;
 //	float dumfl = 0.0;
 
 
+
 	len1 = strlen( seq1[0] );
 	len2 = strlen( seq2[0] );
 	nlentmp = MAX( len1, len2 );
@@ -1786,7 +1863,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 
 		for( j=0; j<n20or4or2; j++ )
 		{
-			fft( nlen, seqVector2[j], (j==0) );
+			fft( nlen, seqVector2[j], 0 );
 			fft( nlen, seqVector1[j], 0 );
 		}
 #if 0
@@ -2158,6 +2235,8 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 	for( i=0; i<count-1; i++ )
 	{
 		*fftlog += 1;
+		if( i == 0 ) headgp = outgap; else headgp = 1;
+		if( i == count-2 ) tailgp = outgap; else tailgp = 1;
 
 		if( cut1[i] )
 		{
@@ -2243,7 +2322,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 		switch( alg )
 		{
 			case( 'M' ):
-					totalscore += MSalignmm( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, sgap1, sgap2, egap1, egap2 );
+					totalscore += MSalignmm( tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, sgap1, sgap2, egap1, egap2, NULL, 0, NULL, headgp, tailgp );
 				break;
 			default:
 				fprintf( stderr, "alg = %c\n", alg );
