@@ -36,7 +36,8 @@ static void match_calc( float *match, char **s1, char **s2, int i1, int lgth2 )
 static float Atracking( float *lasthorizontalw, float *lastverticalw, 
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
-                        int **ijp )
+                        int **ijp,
+						int tailgp )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, limk;
 //	char gap[] = "-";
@@ -44,6 +45,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	gap = newgapstr;
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
+	float wm;
 
 
 #if 0
@@ -62,11 +64,39 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
         ijp[0][j] = -( j + 1 );
     }
 
+	if( tailgp == 1 )
+		;
+	else
+	{
+		wm = lastverticalw[0];
+		for( i=0; i<lgth1; i++ )
+		{
+			if( lastverticalw[i] >= wm )
+			{
+				wm = lastverticalw[i];
+				iin = i; jin = lgth2-1;
+				ijp[lgth1][lgth2] = +( lgth1 - i );
+			}
+		}
+		for( j=0; j<lgth2; j++ )
+		{
+			if( lasthorizontalw[j] >= wm )
+			{
+				wm = lasthorizontalw[j];
+				iin = lgth1-1; jin = j;
+				ijp[lgth1][lgth2] = -( lgth2 - j );
+			}
+		}
+	}
+
+
 
 	mseq1[0] += lgth1+lgth2;
 	*mseq1[0] = 0;
 	mseq2[0] += lgth1+lgth2;
 	*mseq2[0] = 0;
+
+
 	iin = lgth1; jin = lgth2;
 	limk = lgth1+lgth2 + 1;
 	for( k=0; k<limk; k++ ) 
@@ -107,7 +137,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 }
 
 
-float G__align11( char **seq1, char **seq2, int alloclen )
+float G__align11( char **seq1, char **seq2, int alloclen, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -142,6 +172,34 @@ float G__align11( char **seq1, char **seq2, int alloclen )
 	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
 
+	if( seq1 == NULL )
+	{
+		if( orlgth1 > 0 && orlgth2 > 0 )
+		{
+			orlgth1 = 0;
+			orlgth2 = 0;
+			free( mseq1 );
+			free( mseq2 );
+			FreeFloatVec( w1 );
+			FreeFloatVec( w2 );
+			FreeFloatVec( match );
+			FreeFloatVec( initverticalw );
+			FreeFloatVec( lastverticalw );
+
+			FreeFloatVec( m );
+			FreeIntVec( mp );
+
+			FreeCharMtx( mseq );
+
+
+
+			FreeFloatMtx( floatwork );
+			FreeIntMtx( intwork );
+		}
+		return( 0.0 );
+	}
+
+
 	wm = 0.0;
 
 	if( orlgth1 == 0 )
@@ -262,7 +320,7 @@ float G__align11( char **seq1, char **seq2, int alloclen )
 
 	match_calc( currentw, seq1, seq2, 0, lgth2 );
 
-	if( outgap == 1 )
+	if( headgp == 1 )
 	{
 		for( i=1; i<lgth1+1; i++ )
 		{
@@ -284,7 +342,7 @@ float G__align11( char **seq1, char **seq2, int alloclen )
 	else
 		lastverticalw[0] = currentw[lgth2-1]; // lgth2==0 no toki error
 
-	if( outgap ) lasti = lgth1+1; else lasti = lgth1;
+	if( tailgp ) lasti = lgth1+1; else lasti = lgth1;
 
 #if XXXXXXX
 fprintf( stderr, "currentw = \n" );
@@ -393,7 +451,7 @@ fprintf( stderr, "\n" );
 		lastverticalw[i] = currentw[lgth2-1]; // lgth2==0 no toki error
 	}
 
-	Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp );
+	Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, ijp, tailgp );
 
 	resultlen = strlen( mseq1[0] );
 	if( alloclen < resultlen || resultlen > N )
@@ -444,6 +502,26 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 	static TLS float **floatwork;
 	static TLS int orlgth1 = 0, orlgth2 = 0;
 
+	if( seq1 == NULL )
+	{
+		if( orlgth1 > 0 && orlgth2 > 0 )
+		{
+			orlgth1 = 0;
+			orlgth2 = 0;
+			FreeFloatVec( w1 );
+			FreeFloatVec( w2 );
+			FreeFloatVec( match );
+			FreeFloatVec( initverticalw );
+			FreeFloatVec( lastverticalw );
+			free( m );
+
+
+			FreeFloatMtx( floatwork );
+			FreeIntMtx( intwork );
+		}
+		return( 0.0 );
+	}
+
 	wm = 0.0;
 
 
@@ -527,7 +605,7 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 
 	match_calc_mtx( scoremtx, currentw, seq1, seq2, 0, lgth2 );
 
-	if( outgap == 1 )
+	if( 1 ) // tsuneni outgap-1
 	{
 		for( i=1; i<lgth1+1; i++ )
 		{
@@ -549,7 +627,7 @@ float G__align11_noalign( int scoremtx[0x80][0x80], int penal, int penal_ex, cha
 	else
 		lastverticalw[0] = currentw[lgth2-1]; // lgth2==0 no toki error
 
-	if( outgap ) lasti = lgth1+1; else lasti = lgth1;
+	if( 1 ) lasti = lgth1+1; else lasti = lgth1; // tsuneni outgap-1
 
 #if XXXXXXX
 fprintf( stderr, "currentw = \n" );
