@@ -343,7 +343,7 @@ int main( int argc, char *argv[] )
 	LocalHom **localhomtable = NULL; // by D.Mathog
 	RNApair ***singlerna;
 	int nogaplen;
-	char **nogap1seq;
+	static char **nogap1seq;
 	static char *kozoarivec;
 	int nkozo;
 
@@ -441,7 +441,8 @@ int main( int argc, char *argv[] )
 	}
 
 
-		if( nlenmax > 30000 )
+//		if( nlenmax > 30000 )
+		if( nlenmax > 50000 ) // version >= 6.823
 		{
 #if 0
 			if( constraint )
