@@ -34,7 +34,7 @@ char **arguments( int argc, char *argv[] )
 	calledByXced = 0;
 	devide = 0;
 	fftscore = 1;
-	use_fft = 0; // CHUUI!! atode modosu
+	use_fft = 1;
 	alg = 'A';
     weight = 0;
     utree = 1;
@@ -43,7 +43,7 @@ char **arguments( int argc, char *argv[] )
     check = 1;
     cut = 0.0;
     disp = 0;
-    outgap = 1;
+    outgap = 0;
     mix = 0;
 	tbitr = 0;
 	scmtd = 5;
@@ -54,7 +54,7 @@ char **arguments( int argc, char *argv[] )
 	dorp = NOTSPECIFIED;
 	ppenalty = NOTSPECIFIED;
 	ppenalty_ex = NOTSPECIFIED;
-	poffset = NOTSPECIFIED;
+	poffset = 0; // chokusetsu yobareru kara
 	kimuraR = NOTSPECIFIED;
 	pamN = NOTSPECIFIED;
 	fftWinSize = NOTSPECIFIED;
@@ -78,6 +78,9 @@ char **arguments( int argc, char *argv[] )
 				case 'F':
 					use_fft = 1;
 					break;
+				case 'N':
+					use_fft = 0;
+					break;
 				case 'e':
 					fftscore = 0;
 					break;
@@ -96,7 +99,7 @@ char **arguments( int argc, char *argv[] )
 				case 'd':
 					disp = 1;
 					break;
-				case 'o':
+				case 'O':
 					outgap = 0;
 					break;
 				case 'a':
@@ -133,7 +136,7 @@ char **arguments( int argc, char *argv[] )
                     goto nextoption;
                 case 'k':
                     kimuraR = atoi( *++argv );
-                    fprintf( stderr, "kimuraR = %d\n", kimuraR );
+                    fprintf( stderr, "kappa = %d\n", kimuraR );
                     --argc;
                     goto nextoption;
 				case 'b':
@@ -252,18 +255,18 @@ void GroupAlign( int nseq1, int nseq2, char **name, int *nlen, char **seq, char
 		if( alg == 'M' )
 			pscore = Falign_udpari_long( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
 		else
-			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, NULL, 0, NULL );
 	}
 	else
 	{
 		if( alg == 'M' )
-			pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL );
+			pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 		else if( alg == 'H' )
 			pscore = H__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
 		else if( alg == 'Q' )
 			pscore = Q__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
 		else
-			pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
+			pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 	}
 #if DEBUG
 		time2 = getrusage_sec();
@@ -479,7 +482,7 @@ int main( int argc, char *argv[] )
 
 	len1 = strlen( seq[0] );
 	len2 = strlen( seq[nseq1] );
-	if( len1 > 10000 || len2 > 10000 )
+	if( len1 > 30000 || len2 > 30000 )
 	{       
 		fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode.\n", len1, len2 );
 		alg = 'M';
