@@ -83,7 +83,7 @@ static float recallpairfoldalign( char **mseq1, char **mseq2, int m1, int m2, in
 	if( strstr( foldalignopt, "-global") )
 	{
 		fprintf( stderr, "Calling G__align11\n" );
-		value = G__align11( mseq1, mseq2, alloclen );
+		value = G__align11( mseq1, mseq2, alloclen, outgap, outgap );
 		*of1pt = 0;
 		*of2pt = 0;
 	}
@@ -308,7 +308,21 @@ static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, c
 	sprintf( com, "tr -d '\\r' < %s/_mxscarnainorg > %s/_mxscarnain", dirname, dirname );
 	system( com ); // for cygwin, wakaran
 
+#if 0
 	sprintf( com, "cd %s; %s/mxscarnamod -readbpp _mxscarnain > _mxscarnaout 2>_dum", dirname, whereispairalign );
+#else
+	sprintf( com, "_mxscarnash%s", dirname );
+	fp = fopen( com, "w" );
+	fprintf( fp, "cd %s\n", dirname );
+	fprintf( fp, "%s/mxscarnamod -readbpp _mxscarnain > _mxscarnaout 2>_dum\n", whereispairalign );
+	fprintf( fp, "exit $tatus\n" );
+	fclose( fp );
+
+	sprintf( com, "tr -d '\\r' < _mxscarnash%s > _mxscarnash%s.unix", dirname, dirname );
+	system( com ); // for cygwin, wakaran
+
+	sprintf( com, "sh _mxscarnash%s.unix 2>_dum%s", dirname, dirname );
+#endif
 	res = system( com );
 	if( res )
 	{
@@ -337,6 +351,7 @@ static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, c
 
 	value = (float)naivepairscore11( *mseq1, *mseq2, penalty );
 
+#if 0
 	sprintf( com, "rm -rf %s > /dev/null 2>&1", dirname );
 	if( system( com ) )
 	{
@@ -344,6 +359,7 @@ static float callmxscarna_giving_bpp( char **mseq1, char **mseq2, char **bpp1, c
 		usleep( 2000 );
 		system( com );
 	}
+#endif
 
 	free( dirname );
 	free( com );
@@ -806,6 +822,24 @@ static void *athread( void *arg )
 			{
 //				fprintf( stderr, "thread %d end!\n", thread_no );
 				pthread_mutex_unlock( targ->mutex_counter );
+
+				if( commonIP ) FreeIntMtx( commonIP );
+				commonIP = NULL;
+				if( commonJP ) FreeIntMtx( commonJP );
+				commonJP = NULL;
+				Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+				G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
+				L__align11( NULL, NULL, 0, NULL, NULL );
+				genL__align11( NULL, NULL, 0, NULL, NULL );
+				free( effarr );
+				free( effarr1 );
+				free( effarr2 );
+				free( indication1 );
+				free( indication2 );
+				FreeCharMtx( pair  );
+				free( mseq1 );
+				free( mseq2 );
+				FreeCharMtx( aseq  );
 				return( NULL );
 			}
 		}
@@ -850,7 +884,7 @@ static void *athread( void *arg )
 
 		if( use_fft )
 		{
-			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, NULL, 0, NULL );
 //			fprintf( stderr, "pscore (fft) = %f\n", pscore );
 			off1 = off2 = 0;
 		}
@@ -863,7 +897,7 @@ static void *athread( void *arg )
 					L__align11( mseq1, mseq2, alloclen, &off1, &off2 );
 					break;
 				case( 'A' ):
-					pscore = G__align11( mseq1, mseq2, alloclen );
+					pscore = G__align11( mseq1, mseq2, alloclen, outgap, outgap );
 					off1 = off2 = 0;
 					break;
 				case( 'N' ):
@@ -1103,6 +1137,8 @@ static void pairalign( char **name, int nlen[M], char **seq, char **aseq, char *
 		}
 		pthread_mutex_destroy( &mutex_counter );
 		pthread_mutex_destroy( &mutex_stdout );
+		free( handle );
+		free( targ );
 	}
 	else
 #endif
@@ -1140,7 +1176,7 @@ static void pairalign( char **name, int nlen[M], char **seq, char **aseq, char *
 	
 				if( use_fft )
 				{
-					pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+					pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, NULL, 0, NULL );
 //					fprintf( stderr, "pscore (fft) = %f\n", pscore );
 					off1 = off2 = 0;
 				}
@@ -1157,7 +1193,7 @@ static void pairalign( char **name, int nlen[M], char **seq, char **aseq, char *
 							off1 = off2 = 0;
 							break;
 						case( 'A' ):
-							pscore = G__align11( mseq1, mseq2, alloclen );
+							pscore = G__align11( mseq1, mseq2, alloclen, outgap, outgap );
 							off1 = off2 = 0;
 							break;
 						case( 'N' ):
@@ -1289,6 +1325,12 @@ static void pairalign( char **name, int nlen[M], char **seq, char **aseq, char *
 		}
 		free( bpp );
 	}
+	free( selfscore );
+	free( effarr1 );
+	free( effarr2 );
+	free( indication1 );
+	free( indication2 );
+	if( store_dist ) FreeDoubleMtx( distancemtx );
 }
 
 static void WriteOptions( FILE *fp )
@@ -1462,6 +1504,13 @@ int main( int argc, char *argv[] )
 	{
 		fprintf( stderr, "\nThe order of pairwise alignments is not identical to that in the input file, because of the parallel calculation.  Reorder them by yourself.\n" );
 	}
+	FreeCharMtx( seq );
+	FreeCharMtx( aseq );
+	FreeCharMtx( bseq );
+	FreeCharMtx( name );
+	free( mseq1 );
+	free( mseq2 );
+	free( eff );
 
 	return( 0 );
 }
