@@ -101,7 +101,7 @@ static void mccaskillextract( char **seq, char **nogap, int nseq, RNApair **pair
 	int i, j;
 	int left, right, adpos;
 	float prob;
-	static int *pairnum;
+	static TLS int *pairnum;
 	RNApair *pt, *pt2;
 
 	lgth = strlen( seq[0] );
@@ -183,16 +183,16 @@ void rnaalifoldcall( char **seq, int nseq, RNApair **pairprob )
 {
 	int lgth;
 	int i;
-	static int *order = NULL;
-	static char **name = NULL;
+	static TLS int *order = NULL;
+	static TLS char **name = NULL;
 	char gett[1000];
 	FILE *fp;
 	int left, right, dumm;
 	float prob;
-	static int pid;
-	static char fnamein[100];
-	static char cmd[1000];
-	static int *pairnum;
+	static TLS int pid;
+	static TLS char fnamein[100];
+	static TLS char cmd[1000];
+	static TLS int *pairnum;
 
 	lgth = strlen( seq[0] );
 	if( order == NULL )
@@ -331,14 +331,14 @@ void foldrna( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1, doub
 //	int ui, uj;
 //	int uiup, ujup;
 	int uido, ujdo;
-	static char **useq1, **useq2;
-	static char **oseq1, **oseq2, **oseq1r, **oseq2r, *odir1, *odir2;
-	static RNApair **pairprob1, **pairprob2;
-	static RNApair *pairpt1, *pairpt2;
+	static TLS char **useq1, **useq2;
+	static TLS char **oseq1, **oseq2, **oseq1r, **oseq2r, *odir1, *odir2;
+	static TLS RNApair **pairprob1, **pairprob2;
+	static TLS RNApair *pairpt1, *pairpt2;
 	int lgth1 = strlen( seq1[0] );
 	int lgth2 = strlen( seq2[0] );
-	static float **impmtx2;
-	static float **map;
+	static TLS float **impmtx2;
+	static TLS float **map;
 //	double lenfac;
 	float prob;
 	int **sgapmap1, **sgapmap2;
