@@ -1072,7 +1072,7 @@ static void pairalign( int nseq, int *nlen, char **seq, int *mem1, int *mem2, do
 		else
 		{
 //			fprintf( stderr, "%d-%d", clus1, clus2 );
-			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
+			pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1, NULL, 0, NULL );
 		}
 	}
 	else
@@ -1087,13 +1087,13 @@ static void pairalign( int nseq, int *nlen, char **seq, int *mem1, int *mem2, do
 			case( 'M' ):
 				fprintf( stderr, "\bm" );
 //				fprintf( stderr, "%d-%d", clus1, clus2 );
-				pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL );
+				pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 				break;
 			case( 'Q' ):
 				if( clus1 == 1 && clus2 == 1 )
 				{
 //					fprintf( stderr, "%d-%d", clus1, clus2 );
-					pscore = G__align11( mseq1, mseq2, *alloclen );
+					pscore = G__align11( mseq1, mseq2, *alloclen, outgap, outgap );
 				}
 				else
 				{
@@ -1105,12 +1105,12 @@ static void pairalign( int nseq, int *nlen, char **seq, int *mem1, int *mem2, do
 				if( clus1 == 1 && clus2 == 1 )
 				{
 //					fprintf( stderr, "%d-%d", clus1, clus2 );
-					pscore = G__align11( mseq1, mseq2, *alloclen );
+					pscore = G__align11( mseq1, mseq2, *alloclen, outgap, outgap );
 				}
 				else
 				{
 //					fprintf( stderr, "%d-%d", clus1, clus2 );
-					pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
+					pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 				}
 				break;
 			default:
