@@ -223,9 +223,15 @@ void arguments( int argc, char *argv[] )
 				case 'P':
 					dorp = 'p';
 					break;
+#if 1
+				case 'O':
+					outgap = 0;
+					break;
+#else
 				case 'O':
 					fftNoAnchStop = 1;
 					break;
+#endif
 				case 'S':
 					scoreout = 1;
 					break;
@@ -300,11 +306,6 @@ void arguments( int argc, char *argv[] )
 				case 'd':
 					disp = 1;
 					break;
-#if 0
-				case 'o':
-					outgap = 0;
-					break;
-#endif
 /* Modified 01/08/27, default: user tree */
 				case 'J':
 					tbutree = 0;
@@ -535,6 +536,25 @@ static void *treebasethread( void *arg )
 		if( l == njob-1 )
 		{
 			pthread_mutex_unlock( targ->mutex );
+			if( commonIP ) FreeIntMtx( commonIP );
+			commonIP = NULL;
+			Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+			A__align( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, 0, 0 );
+			free( mseq1 );
+			free( mseq2 );
+			free( localcopy );
+			free( effarr1 );
+			free( effarr2 );
+			free( effarr1_kozo );
+			free( effarr2_kozo );
+			free( indication1 );
+			free( indication2 );
+			if( constraint )
+			{
+				for( i=0; i<njob; i++)
+					free( localhomshrink[i] );
+				free( localhomshrink );
+			}
 			return( NULL );
 		}
 		*jobpospt = l+1;
@@ -650,7 +670,7 @@ static void *treebasethread( void *arg )
 		fprintf( stderr, "\n" );
 */
 
-		if( !nevermemsave && ( constraint != 2  && alg != 'M'  && ( len1 > 10000 || len2 > 10000 ) ) )
+		if( !nevermemsave && ( constraint != 2  && alg != 'M'  && ( len1 > 30000 || len2 > 30000 ) ) )
 		{
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode.\n", len1, len2 );
 			alg = 'M';
@@ -678,7 +698,7 @@ static void *treebasethread( void *arg )
 			{
 				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
 				if( rnakozo ) imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, NULL, NULL, NULL );
-				pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumfl, NULL, NULL, NULL, NULL );
+				pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 			}
 			else if( alg == 'H' )
 			{
@@ -706,7 +726,7 @@ static void *treebasethread( void *arg )
 				pscore = Falign_udpari_long( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
 			}
 			else
-				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
+				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1, NULL, 0, NULL );
 		}
 		else
 		{
@@ -719,10 +739,10 @@ static void *treebasethread( void *arg )
 					break;
 				case( 'M' ):
 					fprintf( stderr, "m" );
-					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL );
+					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'A' ):
-					pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
+					pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'Q' ):
 					pscore = Q__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
@@ -798,7 +818,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 	static LocalHom ***localhomshrink = NULL;
 	static int *fftlog;
 	int m1, m2;
-	int *gaplen;
+	static int *gaplen;
 	static int *gapmap;
 	static int *alreadyaligned;
 	float dumfl = 0.0;
@@ -983,7 +1003,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 		fprintf( stderr, "\n" );
 */
 
-		if( !nevermemsave && ( constraint != 2  && alg != 'M'  && ( len1 > 10000 || len2 > 10000 ) ) )
+		if( !nevermemsave && ( constraint != 2  && alg != 'M'  && ( len1 > 30000 || len2 > 30000 ) ) )
 		{
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode.\n", len1, len2 );
 			alg = 'M';
@@ -1011,7 +1031,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			{
 				imp_match_init_strict( NULL, clus1, clus2, strlen( mseq1[0] ), strlen( mseq2[0] ), mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
 				if( rnakozo ) imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, NULL, NULL, NULL );
-				pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumfl, NULL, NULL, NULL, NULL );
+				pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, localhomshrink, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 			}
 			else if( alg == 'H' )
 			{
@@ -1039,7 +1059,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 				pscore = Falign_udpari_long( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
 			}
 			else
-				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
+				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1, NULL, 0, NULL );
 		}
 		else
 		{
@@ -1052,10 +1072,10 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 					break;
 				case( 'M' ):
 					fprintf( stderr, "m" );
-					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL );
+					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'A' ):
-					pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
+					pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'Q' ):
 					pscore = Q__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
@@ -1095,7 +1115,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			adjustgapmap( strlen( mseq2[0] )-len2nocommongap+len2, gapmap, mseq2[0] );
 			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen );
 			findnewgaps( clus2, mseq2, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, *alloclen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][1], topol[l][0], gaplen, gapmap, *alloclen, alg );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
 			for( i=0; (m=topol[l][0][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
@@ -1106,7 +1126,7 @@ void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char **mseq
 			adjustgapmap( strlen( mseq1[0] )-len1nocommongap+len1, gapmap, mseq1[0] );
 			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen );
 			findnewgaps( clus1, mseq1, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, *alloclen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
 			for( i=0; (m=topol[l][1][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
@@ -1414,6 +1434,8 @@ int main( int argc, char *argv[] )
 					pthread_join( handle[i], NULL );
 				}
 				pthread_mutex_destroy( &mutex );
+				free( handle );
+				free( targ );
 			}
 			else
 #endif
@@ -1433,7 +1455,8 @@ int main( int argc, char *argv[] )
 						if( bunbo == 0.0 )
 							iscore[i][j-i] = 1.0;
 						else
-							iscore[i][j-i] = 1.0 - naivepairscore11( seq[i], seq[j], penalty ) / MIN( selfscore[i], selfscore[j] );
+//							iscore[i][j-i] = 1.0 - naivepairscore11( seq[i], seq[j], penalty ) / MIN( selfscore[i], selfscore[j] );
+							iscore[i][j-i] = 1.0 - naivepairscore11( seq[i], seq[j], penalty ) / bunbo;
 		
 #if 0
 						fprintf( stderr, "### ssj = %f\n", ssj );
@@ -1798,6 +1821,9 @@ int main( int argc, char *argv[] )
 		}
 		pthread_mutex_destroy( &mutex );
 		pthread_cond_destroy( &treecond );
+		free( handle );
+		free( targ );
+		free( fftlog );
 	}
 	else
 #endif
