@@ -158,8 +158,69 @@ static void Writeoptions( FILE *fp )
 }
 
 #ifdef enablemultithread
+
+static void freelocalarrays( 
+	float *tscorehistory,
+	RNApair ***grouprna1, RNApair ***grouprna2,
+	RNApair *rnapairboth,
+	char *indication1, char *indication2,
+	double *effarr, double *effarrforlocalhom, double *effarr1, double *effarr2,
+	char **mseq1, char **mseq2,
+	char **localcopy,
+	int *gapmap1, int *gapmap2,
+	double *effarr1_kozo, double *effarr2_kozo, double *effarr_kozo,
+	char **pair,
+	LocalHom *** localhomshrink
+)
+{
+//	fprintf( stderr, "Skipping freelocalarrays\n" );
+//	return;
+	int i;
+	if( commonIP ) FreeIntMtx( commonIP );
+	commonIP = NULL;
+	Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+	Falign_localhom( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, NULL, NULL,NULL, 0, NULL );
+	if( rnakozo && rnaprediction == 'm' )
+	{
+		free( grouprna1 ); // nakamimo?
+		free( grouprna2 ); // nakamimo?
+	}
+
+	free( tscorehistory );
+	free( indication1 );
+	free( indication2 );
+	free( effarr );
+	free( effarrforlocalhom );
+	free( effarr1 );
+	free( effarr2 );
+	free( mseq1 );
+	free( mseq2 );
+	FreeCharMtx( localcopy );
+	free( gapmap1 );
+	free( gapmap2 );
+
+	free( effarr1_kozo );
+	free( effarr2_kozo );
+	free( effarr_kozo );
+
+	FreeCharMtx( pair );
+
+	if( rnakozo ) free( rnapairboth );
+
+	if( constraint )
+	{
+		for( i=0; i<njob; i++)
+		{
+			free( localhomshrink[i] ); // nakamimo??
+		}
+		free( localhomshrink );
+	}
+}
+
+
 static void *athread( void *arg )
 {
+
 	threadarg_t *targ = (threadarg_t *)arg;
 	int thread_no = targ->thread_no;
 	int njob = targ->njob;
@@ -211,7 +272,6 @@ static void *athread( void *arg )
 	double tmpdouble;
 	float naivescore0 = 0, naivescore1;
 	double *effarrforlocalhom;
-	static float ***history;
 	float *tscorehistory;
 	int intdum;
 #if 0
@@ -225,6 +285,7 @@ static void *athread( void *arg )
 	double unweightedspscore;
 	int myjob;
 	int converged2 = 0;
+	int chudanres;
 
 
 	locnjob = njob;
@@ -272,7 +333,6 @@ static void *athread( void *arg )
 	mseq1 = AllocateCharMtx( locnjob, 0 );
 	mseq2 = AllocateCharMtx( locnjob, 0 );
 	localcopy = AllocateCharMtx( locnjob, alloclen );
-	history = AllocateFloatCub( maxiter, locnjob, 2 );
 	gapmap1 = AllocateIntVec( alloclen );
 	gapmap2 = AllocateIntVec( alloclen );
 
@@ -325,7 +385,22 @@ static void *athread( void *arg )
 			{
 				pthread_cond_broadcast( targ->collection_end );
 				pthread_mutex_unlock( targ->mutex );
-				return( NULL );
+				freelocalarrays
+				( 
+					tscorehistory,
+					grouprna1, grouprna2,
+					rnapairboth,
+					indication1, indication2,
+					effarr, effarrforlocalhom, effarr1, effarr2,
+					mseq1, mseq2,
+					localcopy,
+					gapmap1, gapmap2,
+					effarr1_kozo, effarr2_kozo, effarr_kozo,
+					pair,
+					localhomshrink
+				);
+//				return( NULL );
+				pthread_exit( NULL );
 			}
 
 			pthread_mutex_lock( targ->mutex );
@@ -402,7 +477,9 @@ static void *athread( void *arg )
 				*collectingpt = -1;
 //				pthread_cond_broadcast( targ->collection_end );
 //				pthread_mutex_unlock( targ->mutex );
+//				freelocalarrays();
 //				return( NULL );
+//				pthread_exit( NULL );
 			}
 
 #if 1
@@ -420,7 +497,22 @@ static void *athread( void *arg )
 		*collectingpt = -1;
 		pthread_cond_broadcast( targ->collection_end );
 		pthread_mutex_unlock( targ->mutex );
+		freelocalarrays
+		( 
+			tscorehistory,
+			grouprna1, grouprna2,
+			rnapairboth,
+			indication1, indication2,
+			effarr, effarrforlocalhom, effarr1, effarr2,
+			mseq1, mseq2,
+			localcopy,
+			gapmap1, gapmap2,
+			effarr1_kozo, effarr2_kozo, effarr_kozo,
+			pair,
+			localhomshrink
+		);
 		return( NULL );
+		pthread_exit( NULL );
 	}
 	else
 	{
@@ -445,7 +537,22 @@ static void *athread( void *arg )
 			if( *collectingpt == -1 )
 			{
 				pthread_mutex_unlock( targ->mutex );
+				freelocalarrays
+				( 
+					tscorehistory,
+					grouprna1, grouprna2,
+					rnapairboth,
+					indication1, indication2,
+					effarr, effarrforlocalhom, effarr1, effarr2,
+					mseq1, mseq2,
+					localcopy,
+					gapmap1, gapmap2,
+					effarr1_kozo, effarr2_kozo, effarr_kozo,
+					pair,
+					localhomshrink
+				);
 				return( NULL );
+				pthread_exit( NULL );
 			}
 //			pthread_mutex_unlock( targ->mutex );
 
@@ -686,8 +793,14 @@ static void *athread( void *arg )
 //						part_imp_match_init_strictQ( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, localhomshrink, 1 );
 //					else
 //						part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
-					Falign_localhom( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, localhomshrink, &impmatch, gapmap1, gapmap2 );
+					chudanres = 0;
+					Falign_localhom( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, localhomshrink, &impmatch, gapmap1, gapmap2, subgenerationpt, subgenerationatfirst, &chudanres );
 //					fprintf( stderr, "##### impmatch = %f\n", impmatch );
+					if( chudanres && parallelizationstrategy == BAATARI2 )
+					{
+//						fprintf( stderr, "#### yarinaoshi!!! INS-i\n" );
+						goto yarinaoshi;
+					}
 				}
 				else
 				{
@@ -777,6 +890,7 @@ static void *athread( void *arg )
 					{
 //						imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
 						A__align_gapmap( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, localhomshrink, &impmatch, gapmap1, gapmap2 );
+						fprintf( stderr, "A__align_gapmap\n" );
 //						fprintf( stderr, "##### impmatch = %f\n", impmatch );
 					}
 				}
@@ -784,7 +898,13 @@ static void *athread( void *arg )
 			else if( use_fft )
 			{
 				float totalwm;
-				totalwm = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+				chudanres = 0;
+				totalwm = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, subgenerationpt, subgenerationatfirst, &chudanres );
+				if( chudanres && parallelizationstrategy == BAATARI2 )
+				{
+//					fprintf( stderr, "#### yarinaoshi!!! FFT-NS-i\n" );
+					goto yarinaoshi;
+				}
 
 //				fprintf( stderr, "totalwm = %f\n", totalwm );
 #if 0
@@ -836,11 +956,17 @@ static void *athread( void *arg )
 			{
 				if( alg == 'M' )
 				{
-					MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL );
+					chudanres = 0;
+					MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL, subgenerationpt, subgenerationatfirst, &chudanres, outgap, outgap );
+					if( chudanres && parallelizationstrategy == BAATARI2 )
+					{
+//						fprintf( stderr, "#### yarinaoshi!!! NW-NS-i\n" );
+						goto yarinaoshi;
+					}
 				}
 				else if( alg == 'A' )
 				{
-					A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &impmatch, NULL, NULL, NULL, NULL );
+					A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &impmatch, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1 ); //outgap==1
 				}
 				else if( alg == 'Q' )
 				{
@@ -1101,7 +1227,7 @@ static void *athread( void *arg )
 		}            /* while( 1 ) */
 
 	}                  /* for( iterate ) */
-	return( NULL );
+//	return( NULL );
 }
 #endif
 
@@ -1331,9 +1457,20 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 		{
 			pthread_join( handle[i], NULL );
 		}
+
 		pthread_mutex_destroy( &mutex );
 		pthread_cond_destroy( &collection_end );
 		pthread_cond_destroy( &collection_start );
+
+		free( targ );
+		free( handle );
+		free( gainlist );
+		free( tscorelist );
+		free( branchtable );
+		free( generationofinput );
+		if( parallelizationstrategy == BESTFIRST )
+			FreeCharCub( candidates );
+		FreeFloatMtx( tscorehistory_detail );
 	}
 	else
 #endif
@@ -1832,7 +1969,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 	//							part_imp_match_init_strictQ( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, localhomshrink, 1 );
 	//						else
 	//							part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
-							Falign_localhom( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, localhomshrink, &impmatch, gapmap1, gapmap2 );
+							Falign_localhom( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, localhomshrink, &impmatch, gapmap1, gapmap2, NULL, 0, NULL );
 	//						fprintf( stderr, "##### impmatch = %f\n", impmatch );
 						}
 						else
@@ -1930,7 +2067,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 					else if( use_fft )
 					{
 						float totalwm;
-						totalwm = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum );
+						totalwm = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, &intdum, NULL, 0, NULL );
 	
 	//					fprintf( stderr, "totalwm = %f\n", totalwm );
 	#if 0
@@ -1982,11 +2119,11 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 					{
 						if( alg == 'M' )
 						{
-							MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL );
+							MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 						}
 						else if( alg == 'A' )
 						{
-							A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &impmatch, NULL, NULL, NULL, NULL );
+							A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &impmatch, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1 ); // outgap==1
 						}
 						else if( alg == 'Q' )
 						{
